<?php

declare(strict_types=1);

namespace Drupal\reviewer\Reviewer\Builder;

use Drupal\Core\Config\ConfigManagerInterface;
use Drupal\Core\Config\Entity\ConfigEntityInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\reviewer\Plugin\reviewer\Review\EntityReviewPluginInterface;
use Drupal\reviewer\Plugin\reviewer\Review\ReviewPluginInterface;

/**
 * Builds arrays of configuration and configuration entities.
 */
final readonly class ConfigBuilder implements ConfigBuilderInterface {

  // phpcs:ignore Drupal.Commenting.FunctionComment.Missing
  public function __construct(
    private ConfigManagerInterface $configManager,
    private EntityTypeManagerInterface $entityTypeManager,
  ) {}

  /**
   * {@inheritdoc}
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function configurationEntities(
    ReviewPluginInterface $review,
    string $bundle = '',
  ): array {
    $configuration = [];

    foreach ($review->getConfiguration() as $id) {
      $config = $this->configManager->getConfigFactory()->get($id);
      $configuration[$id] = $config->getRawData()
        ? $config
        : $this->configManager->loadConfigEntityByName($id);
    }

    if ($review instanceof EntityReviewPluginInterface) {
      $configuration += $bundle
        ? $this->bundleConfiguration($review, $bundle)
        : throw new \InvalidArgumentException(sprintf(
          'Parameter $bundle must be a non-empty string when parameter $review is an instance of %s.',
          EntityReviewPluginInterface::class,
        ));
    }

    return $configuration;
  }

  /**
   * Load configuration entities based on the entity review plugin's attributes.
   *
   * @return array<string, \Drupal\Core\Config\Entity\ConfigEntityInterface>
   *   Configuration keyed by the config ID.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  private function bundleConfiguration(
    EntityReviewPluginInterface $review,
    string $bundle,
  ): array {
    $config = $this
      ->entityTypeManager
      ->getStorage($review->getEntityType())
      ->load($bundle);

    if (!$config) {
      return [];
    }

    assert($config instanceof ConfigEntityInterface);

    $configuration[(string) $config->id()] = $config;

    foreach ($this->dependentConfiguration($config) as $dependent_config) {
      $configuration[$dependent_config->getConfigDependencyName()] = $dependent_config;
    }

    return $configuration;
  }

  /**
   * Load dependent configuration entities.
   *
   * @return array<string, \Drupal\Core\Config\Entity\ConfigEntityInterface>
   *   Configuration keyed by the config entity ID.
   */
  private function dependentConfiguration(EntityInterface $entity): array {
    return $entity instanceof ConfigEntityInterface
      ? $this->configManager->findConfigEntityDependenciesAsEntities(
        'config',
        [$entity->getConfigDependencyName()],
      )
      : [];
  }

}
