<?php

declare(strict_types=1);

namespace Drupal\Tests\reviewer\Kernel\Builder;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ConfigManagerInterface;
use Drupal\Core\Config\Entity\ConfigEntityInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\KernelTests\KernelTestBase;
use Drupal\reviewer\Reviewer\Builder\ConfigBuilder;
use Drupal\reviewer\Reviewer\Builder\ConfigBuilderInterface;
use Drupal\Tests\reviewer\Traits\MockReviewPluginTrait;

/**
 * Test the builder.
 *
 * @coversDefaultClass \Drupal\reviewer\Reviewer\Builder\Review\ReviewBuilder
 *
 * @group reviewer
 */
final class ConfigBuilderTest extends KernelTestBase {

  use MockReviewPluginTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'reviewer',
  ];

  /**
   * Test loading configuration from a review plugin.
   *
   * @throws \Exception
   * @throws \ReflectionException
   * @throws \InvalidArgumentException
   * @throws \Symfony\Component\DependencyInjection\Exception\ServiceCircularReferenceException
   * @throws \Symfony\Component\DependencyInjection\Exception\ServiceNotFoundException
   */
  public function testConfigurationEntities(): void {
    $review_plugin = $this->mockReviewPlugin(configuration: ['config_id']);
    $config = $this
      ->getMockBuilder(ImmutableConfig::class)
      ->disableOriginalConstructor()
      ->getMock();
    $config
      ->method('getRawData')
      ->willReturn(['config']);
    $config_empty = $this
      ->getMockBuilder(ImmutableConfig::class)
      ->disableOriginalConstructor()
      ->getMock();
    $config_empty
      ->method('getRawData')
      ->willReturn([]);
    $config_entity = $this
      ->getMockBuilder(ConfigEntityInterface::class)
      ->disableOriginalConstructor()
      ->getMock();

    // Test when \Drupal\Core\Config\ImmutableConfig::getRawData returns data.
    $config_factory = $this
      ->getMockBuilder(ConfigFactoryInterface::class)
      ->disableOriginalConstructor()
      ->getMock();
    $config_factory
      ->method('get')
      ->willReturn($config);
    $config_manager = $this
      ->getMockBuilder(ConfigManagerInterface::class)
      ->disableOriginalConstructor()
      ->getMock();
    $config_manager
      ->method('getConfigFactory')
      ->willReturn($config_factory);
    $this->container->set('config.manager', $config_manager);
    $builder = $this->configBuilder();
    $configurationEntities = new \ReflectionMethod($builder, 'configurationEntities');

    self::assertSame(
      ['config_id' => $config],
      $configurationEntities->invoke($builder, $review_plugin),
    );

    // Test when \Drupal\Core\Config\ImmutableConfig::getRawData does not
    // return data and a config entity is loaded.
    $config_factory = $this
      ->getMockBuilder(ConfigFactoryInterface::class)
      ->disableOriginalConstructor()
      ->getMock();
    $config_factory
      ->method('get')
      ->willReturn($config_empty);
    $config_manager = $this
      ->getMockBuilder(ConfigManagerInterface::class)
      ->disableOriginalConstructor()
      ->getMock();
    $config_manager
      ->method('getConfigFactory')
      ->willReturn($config_factory);
    $config_manager
      ->method('loadConfigEntityByName')
      ->willReturn($config_entity);
    $this->container->set('config.manager', $config_manager);
    $builder = $this->configBuilder();
    $configurationEntities = new \ReflectionMethod($builder, 'configurationEntities');

    self::assertSame(
      ['config_id' => $config_entity],
      $configurationEntities->invoke($builder, $review_plugin),
    );

  }

  /**
   * Test loading configuration entities from an entity review plugin.
   *
   * @throws \Exception
   * @throws \ReflectionException
   * @throws \InvalidArgumentException
   * @throws \Symfony\Component\DependencyInjection\Exception\ServiceCircularReferenceException
   * @throws \Symfony\Component\DependencyInjection\Exception\ServiceNotFoundException
   */
  public function testBundleConfiguration(): void {
    $entity_review_plugin = $this->mockEntityReviewPlugin(
      label: 'Entity Review',
      entity_type: 'node_type',
    );

    // Test returning an empty array when null is returned by
    // \Drupal\Core\Entity\EntityTypeManagerInterface::load.
    $entity_storage = $this
      ->getMockBuilder(EntityStorageInterface::class)
      ->disableOriginalConstructor()
      ->getMock();
    $entity_storage
      ->method('load')
      ->willReturn(NULL);
    $entity_type_manager = $this
      ->getMockBuilder(EntityTypeManagerInterface::class)
      ->disableOriginalConstructor()
      ->getMock();
    $entity_type_manager
      ->method('getStorage')
      ->willReturn($entity_storage);
    $this->container->set('entity_type.manager', $entity_type_manager);
    $builder = $this->configBuilder();
    $bundleConfiguration = new \ReflectionMethod($builder, 'bundleConfiguration');

    self::assertSame(
      [],
      $bundleConfiguration->invoke($builder, $entity_review_plugin, 'bundle'),
    );

    // Test when a config entity is returned by
    // Drupal\Core\Entity\EntityTypeManagerInterface::load.
    $config_entity = $this
      ->getMockBuilder(ConfigEntityInterface::class)
      ->getMock();
    $config_entity
      ->method('id')
      ->willReturn('config_id');
    $entity_storage = $this
      ->getMockBuilder(EntityStorageInterface::class)
      ->disableOriginalConstructor()
      ->getMock();
    $entity_storage
      ->method('load')
      ->willReturn($config_entity);
    $entity_type_manager = $this
      ->getMockBuilder(EntityTypeManagerInterface::class)
      ->disableOriginalConstructor()
      ->getMock();
    $entity_type_manager
      ->method('getStorage')
      ->willReturn($entity_storage);
    $this->container->set('entity_type.manager', $entity_type_manager);
    $builder = $this->configBuilder();
    $bundleConfiguration = new \ReflectionMethod($builder, 'bundleConfiguration');

    self::assertSame(
      ['config_id' => $config_entity],
      $bundleConfiguration->invoke($builder, $entity_review_plugin, 'bundle'),
    );
  }

  /**
   * Test loading all dependent configuration of a configuration entity.
   *
   * @throws \Exception
   * @throws \ReflectionException
   * @throws \InvalidArgumentException
   * @throws \Symfony\Component\DependencyInjection\Exception\ServiceCircularReferenceException
   * @throws \Symfony\Component\DependencyInjection\Exception\ServiceNotFoundException
   */
  public function testDependentConfiguration(): void {
    $config = $this
      ->getMockBuilder(ConfigManagerInterface::class)
      ->disableOriginalConstructor()
      ->getMock();
    $config
      ->method('findConfigEntityDependenciesAsEntities')
      ->willReturn(['one', 'two', 'three']);
    $config_entity = $this
      ->getMockBuilder(ConfigEntityInterface::class)
      ->getMock();
    $this->container->set('config.manager', $config);

    $builder = $this->configBuilder();
    $dependentConfiguration = new \ReflectionMethod($builder, 'dependentConfiguration');

    self::assertSame(
      ['one', 'two', 'three'],
      $dependentConfiguration->invoke($builder, $config_entity),
    );
  }

  /**
   * Get the config builder.
   *
   * @throws \Exception
   * @throws \InvalidArgumentException
   * @throws \Symfony\Component\DependencyInjection\Exception\ServiceCircularReferenceException
   * @throws \Symfony\Component\DependencyInjection\Exception\ServiceNotFoundException
   */
  private function configBuilder(): ConfigBuilderInterface {
    return new ConfigBuilder(
      $this->container->get('config.manager'),
      $this->container->get('entity_type.manager'),
    );
  }

}
