<?php

declare(strict_types=1);

namespace Drupal\Tests\reviewer\Kernel\Builder;

use DG\BypassFinals;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\KernelTests\KernelTestBase;
use Drupal\reviewer\Plugin\reviewer\Review\EntityReviewPluginInterface;
use Drupal\reviewer\Plugin\ReviewManagerInterface;
use Drupal\reviewer\Reviewer\Builder\ConfigBuilderInterface;
use Drupal\reviewer\Reviewer\Builder\Review\ReviewBuilderInterface;
use Drupal\reviewer\Reviewer\Result\ResultFactory;
use Drupal\reviewer\Reviewer\Review\ReviewInterface;
use Drupal\Tests\reviewer\Traits\MockReviewPluginTrait;
use Drupal\Tests\reviewer\Traits\MockTaskTrait;

/**
 * Tests the review builder.
 *
 * @coversDefaultClass \Drupal\reviewer\Reviewer\Builder\Review\ReviewBuilder
 *
 * @group reviewer
 */
final class ReviewBuilderTest extends KernelTestBase {

  use MockReviewPluginTrait;
  use MockTaskTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'reviewer',
  ];

  private readonly ReviewBuilderInterface $builder;

  /**
   * Test review plugins.
   *
   * @var array<string, \Drupal\reviewer\Plugin\reviewer\Review\ReviewPluginInterface>
   */
  private readonly array $reviews;

  /**
   * {@inheritdoc}
   *
   * @throws \Exception
   */
  protected function setUp(): void {
    BypassFinals::enable();
    parent::setUp();

    $this->reviews = [
      'one' => $this->mockReviewPlugin(
        plugin_id: 'one',
        label: 'One',
      ),
      'two' => $this->mockReviewPlugin(
        plugin_id: 'two',
        label: 'Two',
      ),
      'three' => $this->mockEntityReviewPlugin(
        plugin_id: 'three',
        label: 'Three',
        entity_type: 'type',
        bundles: ['a', 'b'],
      ),
    ];

    $entity = $this
      ->getMockBuilder(EntityInterface::class)
      ->disableOriginalConstructor()
      ->getMock();
    $entity
      ->method('label')
      ->willReturn('Entity Label');
    $entity_storage = $this
      ->getMockBuilder(EntityStorageInterface::class)
      ->disableOriginalConstructor()
      ->getMock();
    $entity_storage
      ->method('load')
      ->willReturn($entity);
    $entity_type_manager = $this
      ->getMockBuilder(EntityTypeManagerInterface::class)
      ->disableOriginalConstructor()
      ->getMock();
    $entity_type_manager
      ->method('getStorage')
      ->willReturn($entity_storage);
    $this->container->set('entity_type.manager', $entity_type_manager);

    $config_builder = $this
      ->getMockBuilder(ConfigBuilderInterface::class)
      ->disableOriginalConstructor()
      ->getMock();
    $config_builder
      ->method('configurationEntities')
      ->willReturn([]);
    $this->container->set('reviewer.builder.config', $config_builder);

    $this->container->set(
      'reviewer.factory.result',
      $this->createMock(ResultFactory::class),
    );

    $definitions = [];
    foreach ($this->reviews as $review) {
      $definitions[$review->getPluginId()] = [
        'id' => $review->getPluginId(),
        'label' => $review->getLabel(),
        'bundles' => $review instanceof EntityReviewPluginInterface
          ? $review->getBundles()
          : [],
      ];
    }
    $plugin_manager = $this
      ->getMockBuilder(ReviewManagerInterface::class)
      ->disableOriginalConstructor()
      ->getMock();
    $plugin_manager
      ->method('getDefinition')
      ->willReturnOnConsecutiveCalls(...$definitions);
    $plugin_manager
      ->method('createInstance')
      ->willReturnOnConsecutiveCalls(...$this->reviews);
    $this->container->set('plugin.manager.reviewer.review', $plugin_manager);

    $this->builder = $this->container->get('reviewer.builder.review');
  }

  /**
   * Test building a review from a plugin ID.
   *
   * @covers ::fromId
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   * @throws \Drupal\reviewer\Exception\CannotOverridePropertyException
   */
  public function testFromId(): void {
    $review = $this->builder->fromId($this->reviews['one']->getPluginId());
    self::assertSame(['one'], array_keys($review));
    self::assertContainsOnlyInstancesOf(ReviewInterface::class, $review);

    $review = $this->builder->fromId($this->reviews['two']->getPluginId());
    self::assertSame(['two'], array_keys($review));
    self::assertContainsOnlyInstancesOf(ReviewInterface::class, $review);

    $reviews = $this->builder->fromId($this->reviews['three']->getPluginId());
    self::assertSame(['three.a', 'three.b'], array_keys($reviews));
    self::assertContainsOnlyInstancesOf(ReviewInterface::class, $review);
  }

  /**
   * Test building a review from an array of plugin IDs.
   *
   * @covers ::fromIds
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   * @throws \Drupal\reviewer\Exception\CannotOverridePropertyException
   */
  public function testFromIds(): void {
    $reviews = $this->builder->fromIds(array_keys($this->reviews));
    self::assertSame(['one', 'two', 'three.a', 'three.b'], array_keys($reviews));
    self::assertContainsOnlyInstancesOf(ReviewInterface::class, $reviews);
  }

  /**
   * Test building a review from a plugin.
   *
   * @covers ::fromPlugin
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   * @throws \Drupal\reviewer\Exception\CannotOverridePropertyException
   */
  public function testFromPlugin(): void {
    $review = $this->builder->fromPlugin($this->reviews['one']);
    self::assertSame(['one'], array_keys($review));
    self::assertContainsOnlyInstancesOf(ReviewInterface::class, $review);

    $reviews = $this->builder->fromPlugin($this->reviews['three']);
    self::assertSame(['three.a', 'three.b'], array_keys($reviews));
    self::assertContainsOnlyInstancesOf(ReviewInterface::class, $reviews);
  }

  /**
   * Test building a review from an array of plugins.
   *
   * @covers ::fromPlugins
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   * @throws \Drupal\reviewer\Exception\CannotOverridePropertyException
   */
  public function testFromPlugins(): void {
    $reviews = $this->builder->fromPlugins($this->reviews);
    self::assertSame(['one', 'two', 'three.a', 'three.b'], array_keys($reviews));
    self::assertContainsOnlyInstancesOf(ReviewInterface::class, $reviews);
  }

  /**
   * Test generating a review label with a bundle name.
   *
   * @covers ::labelWithBundle
   *
   * @throws \ReflectionException
   */
  public function testLabelWithBundle(): void {
    $reflection = new \ReflectionClass($this->builder);
    $labelWithBundle = $reflection->getMethod('labelWithBundle');
    self::assertSame(
      'Three: Entity Label',
      (string) $labelWithBundle->invoke($this->builder, $this->reviews['three'], 'bundle'),
    );
  }

}
