<?php

declare(strict_types=1);

namespace Drupal\Tests\reviewer\Unit\Task;

use DG\BypassFinals;
use Drupal\reviewer\Exception\ChecklistItemReturnException;
use Drupal\reviewer\Reviewer\Task\Task;
use Drupal\Tests\reviewer\Traits\MockTaskTrait;
use Drupal\Tests\UnitTestCase;

/**
 * Test task methods that do not require Drupal to be loaded.
 *
 * @coversDefaultClass \Drupal\reviewer\Reviewer\Task\Task
 *
 * @ingroup reviewer
 */
final class TaskTest extends UnitTestCase {

  use MockTaskTrait;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    BypassFinals::enable();
    parent::setUp();
  }

  /**
   * Test result normalization.
   *
   * @covers ::normalizeResult
   *
   * @throws \ReflectionException
   */
  public function testNormalizeResult(): void {
    $task = $this->mockTask(id: 'one', check: fn() => TRUE);
    $reflection = new \ReflectionClass(Task::class);
    $normalizeResult = $reflection->getMethod('normalizeResult');

    self::assertSame(
      ['one' => TRUE],
      $normalizeResult->invoke($task, TRUE),
    );
    self::assertSame(
      ['one' => FALSE],
      $normalizeResult->invoke($task, FALSE),
    );
    self::assertSame(
      ['one' => TRUE, 'two' => FALSE],
      $normalizeResult->invoke($task, ['one' => TRUE, 'two' => FALSE]),
    );
    $this->expectException(ChecklistItemReturnException::class);
    $normalizeResult->invoke($task, [
      'one' => 1,
      'two' => 'string',
      'three' => new \stdClass(),
    ]);
  }

  /**
   * Test full qualified name generation.
   *
   * @covers ::checkFullQualifiedName
   *
   * @throws \ReflectionException
   */
  public function testCheckFullQualifiedName(): void {
    $task = $this->mockTask(id: 'one', check: $this->returnTrue(...));
    $reflection = new \ReflectionClass(Task::class);
    $checkFullQualifiedName = $reflection->getMethod('checkFullQualifiedName');

    self::assertSame(
      $this::class . '::returnTrue()',
      $checkFullQualifiedName->invoke($task),
    );
  }

  /**
   * @covers ::failureMessageWithIds
   */
  public function testFailureMessageWithIds(): void {
    $task = $this->mockTask(id: 'one', failure_message: 'Failed with @failures.');
    $reflection = new \ReflectionClass(Task::class);
    $failureMessageWithIds = $reflection->getMethod('failureMessageWithIds');

    self::assertSame(
      'Failed with one.',
      $failureMessageWithIds->invoke($task, 'one'),
    );
    self::assertSame(
      'Failed with one.',
      $failureMessageWithIds->invoke($task, ['one']),
    );
    self::assertSame(
      'Failed with one, two, three.',
      $failureMessageWithIds->invoke($task, ['one', 'two', 'three']),
    );
    self::assertSame(
      'Failed with .',
      $failureMessageWithIds->invoke($task, ''),
    );
    self::assertSame(
      'Failed with .',
      $failureMessageWithIds->invoke($task, []),
    );

    $task = $this->mockTask(id: 'one', failure_message: 'Failed');
    $reflection = new \ReflectionClass(Task::class);
    $failureMessageWithIds = $reflection->getMethod('failureMessageWithIds');
    self::assertSame(
      'Failed',
      $failureMessageWithIds->invoke($task, 'one'),
    );
    self::assertSame(
      'Failed',
      $failureMessageWithIds->invoke($task, ['one']),
    );
    self::assertSame(
      'Failed',
      $failureMessageWithIds->invoke($task, ['one', 'two', 'three']),
    );
    self::assertSame(
      'Failed',
      $failureMessageWithIds->invoke($task, ''),
    );
    self::assertSame(
      'Failed',
      $failureMessageWithIds->invoke($task, []),
    );
  }

  /**
   * Check callable that returns true.
   */
  public function returnTrue(): true {
    return TRUE;
  }

}
